/*
 * A boring chat application that illustrates a few Actor concepts:
 * Defining an Actor, starting an Actor, sending messages to an
 * Actor, and processing messages.  
 */

import scala.actors._       // The wildcard "_" means import all members
import scala.actors.Actor._ // Import all class members

// Messages are represented as case classes
// For more info on case classes, see http://www.scala-lang.org/node/107
case class User(name: String)
case class Subscribe(user: User)
case class Unsubscribe(user: User)
case class Post(msg: String)
case class PostFrom(user: User, post: Post)

// The chatroom is a kind of Actor
class ChatRoom extends Actor { 

  // Define the act() method
  def act() { 

    // A session for clients that maps a user to an internal Actor
	// representation of the user
    var session = Map.empty[User, Actor] 

	// Loop to start processing messages
    while (true) {

      receive { // Receive the next message from the mailbox

		// Process a Subscribe message by creating a new Actor that will
		// represent the user in the chatroom
        case Subscribe(user) =>
          val subscriberUser = actor {
            while (true) {
              self.receive { // self refers the "inside" actor created
			  				 // by the actor{...} method

                case Post(msg) => Console.println(msg)
              } 
            }
          }
		  // Add the subscriber to the session
          session = session + (user -> subscriberUser)

		// Process a PostFrom messages by resending it to everyone
		// In this example, there's only one subscriber, so this will 
		// simply echo the message to the standard output
        case PostFrom(user, msg) =>
          for (k <- session.keys) {
            session(k) ! msg
          }
      }
    }
  }
}

object Chat {
  def main(args: Array[String]) {
	require(args.length == 1)
	val user = args(0)

    val chatroom = new ChatRoom
    chatroom.start()
    chatroom ! Subscribe(User(user))

	while (true)  {
    	chatroom ! PostFrom(User(user), Post(readLine))
	}
  }
}
