/*
 * A chat program that waits for a subscription acknowledgement from the
 * chat server, using the !? (synchronous message send) operator
 */
import scala.actors._
import scala.actors.Actor._

case class User(name: String)
case class Subscribe(user: User)
case class Unsubscribe(user: User)
case class Post(msg: String)
case class PostFrom(user: User, post: Post)

class ChatRoom extends Actor {

  def act() {
    var session = Map.empty[User, Actor]
    while (true) {
      receive {
        case Subscribe(user) =>
		  Thread.sleep(5000) // Introduce some latency in the 
		  					 // subscription message processing
          val subscriberUser = actor {
            while (true) {
              self.receiveWithin(10 * 1000) {
                case Post(msg) =>
                  println(msg)
				case TIMEOUT =>
				   this ! Unsubscribe(user)
				   exit()
              }
            }
          }
          session = session + (user -> subscriberUser)

		  reply("Subscribed " + user) // Send back a reply
		  							  // The client is waiting for this
									  // reply

        case PostFrom(user, msg) =>
          for (k <- session.keys) {
            session(k) ! msg
          }

		case Unsubscribe(user) =>
		  if (session.contains(user)) {
		    session  -= user
		    println("Unsubscribed " + user)
		  }
      }
    }
  }
}

object SynchronousChat {
  def main(args: Array[String]) {
	require(args.length == 1)
	val user = args(0)

    val chatroom = new ChatRoom
    chatroom.start()

	println("Sending subscription request for " + user + "...")

	// The !? operator will block the current thread until a reply
	// is received. We use pattern matching here to evaluate the reply.
    chatroom !? Subscribe(User(user)) match {
		case response: String => println(response)
	}

	while (true)  {
		val r = readLine
    	chatroom ! PostFrom(User(user), Post(r))
	}
  }
}
